require 'omniauth-oauth'
require 'active_support/core_ext/hash'
require 'naranya_id'
require 'oauth/signature/plaintext'

module OmniAuth
  module Strategies
    class NaranyaId < OmniAuth::Strategies::OAuth
      # Give your strategy a name.
      option :name, "naranya_id"

      # This is where you pass the options you would pass when
      # initializing your consumer from the OAuth gem.
      option :client_options, ::NaranyaId.client_options

      # Override para habilitar/deshabilitar el output de debuggeo de HTTP:
      def consumer
        consumer = ::OAuth::Consumer.new(options.consumer_key, options.consumer_secret, options.client_options)
        consumer.http.open_timeout = options.open_timeout if options.open_timeout
        consumer.http.read_timeout = options.read_timeout if options.read_timeout
        
        # Se habilita el output sólo si en las opciones debug_oauth es true:
        consumer.http.set_debug_output($stderr)           if options.debug_oauth
        consumer
      end

      # Override de método `request_phase` para enviar el parámetro de lang/locale:
      def request_phase
        request_token = consumer.get_request_token({:oauth_callback => callback_url}, options.request_params)
        session['oauth'] ||= {}
        session['oauth'][name.to_s] = {'callback_confirmed' => request_token.callback_confirmed?, 'request_token' => request_token.token, 'request_secret' => request_token.secret}

        # Setup de parametros de lang/locale:
        authorize_params = options[:authorize_params].merge locale_params

        if request_token.callback_confirmed?
          redirect request_token.authorize_url authorize_params
        else
          redirect request_token.authorize_url(authorize_params.merge(:oauth_callback => callback_url))
        end

      rescue ::Timeout::Error => e
        fail!(:timeout, e)
      rescue ::Net::HTTPFatalError, ::OpenSSL::SSL::SSLError => e
        fail!(:service_unavailable, e)
      end

      # These are called after authentication has succeeded. If
      # possible, you should try to set the UID without making
      # additional calls (if the user id is returned with the token
      # or as a URI parameter). This may not be possible with all
      # providers.
      uid do
        raw_info[:id]
      end

      info do
        {
          uid:      raw_info[:id],
          nickname: raw_info[:screen_name],
          name:     "#{raw_info[:first_name]} #{raw_info[:last_name]}"
        }
      end

      extra do
        { 'raw_info' => raw_info }.with_indifferent_access
      end

      # Provee la informacion consultada del usuario vía el API de Naranya ID ()
      def raw_info
        @raw_info ||= begin
          configure_sdk! if sdk_key_or_secret_missing? and consumer_key_and_secret_present?
          u = ::NaranyaId::User.find_one(
            token:  access_token.params[:oauth_token],
            secret: access_token.params[:oauth_token_secret]
          )
          u.attributes.with_indifferent_access
        rescue ::Errno::ETIMEDOUT
          raise ::Timeout::Error
        end
      end

      # Indicates if the SDK's api key + secret are missing
      def sdk_key_or_secret_missing?
        !(::NaranyaId.consumer_key && ::NaranyaId.consumer_secret)
      end

      # Indicates whether the consumer key and secret where configured in the strategy initializer
      def consumer_key_and_secret_present?
        options.consumer_key && options.consumer_secret
      end

      # Configures the SDK's key + secret using the strategy's configured key and secret
      def configure_sdk!
        ::NaranyaId.configure do |config|
          %w(consumer_key consumer_secret debug_oauth).each do |method|
            config.send "#{method}=".to_sym, options.send(method.to_sym)
          end
        end
      end

      private

      def locale_params
        _loc_params = {}

        params = request.params.with_indifferent_access

        client_locale = if params[:locale].present?
          params[:locale]
        end
        
        if client_locale.present?
          # If the complete locale is not available, we'll use the language part of the client locale:
          client_language = client_locale.split('-').first

          # User the client_locale or the client_language, if the former is not available:
          if defined?(I18n) && I18n.respond_to?(:locale_available?) && I18n.respond_to?(:locale)
            if I18n.locale_available? client_locale
              _loc_params[:locale] = client_locale
              _loc_params[:lang] = client_language
              OmniAuth.logger.debug "The client's locale '#{client_locale}' is available in the app, and will be sent to OAuth site"
            else
              OmniAuth.logger.debug "The client's locale '#{client_locale}' is not available in the app, and will not be sent to OAuth site"
            end
          end

        end

        _loc_params

      end

    end
  end
end